/*
 * Main - Sample program demonstrating the use of plugins
 * Copyright (C) 2006  Johannes Mayer
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

package examples;

import de.uulm.plugin.*;
import java.util.*;

/**
* Minimal demo on how to use plugins.
*
* @author  Johannes Mayer
* @version 0.2, 2006-12-05
*/
@SuppressWarnings("unchecked")
public class Main {
    public static void main(String[] args) {
	//*****************************************************************
	// part 1: unconditional invocation of all plugins
	//*****************************************************************

        // get plugin registry
        PluginRegistry reg = PluginRegistry.getInstance();

	// retrieve all print plugins
	PrintPlugin[] ps = (PrintPlugin[]) reg.getPlugins(PrintPlugin.class);

	// iterate over all print plugins (in non-deterministic order!)
	// and work with them
	for (PrintPlugin p : ps)
	    p.print();

	//*****************************************************************
	// part 2: conditional invocation of plugins (using voting methods)
	//*****************************************************************

	// The objects in the following list should be displayed using plugins
	List l = new Vector();
	l.add("Test");
	l.add(3.14);
	l.add(3);

	DisplayPlugin[] ds = (DisplayPlugin[]) reg.getPlugins(DisplayPlugin.class);

	// iterate over all element in l which are to be displayed and
	// iterate over all display plugins (in non-deterministic order!)
	// ... take the first plugin (that is able to display o)
	for (Object o : l) {
	    boolean displayed = false;
	    for (DisplayPlugin p : ds)
	        if (p.canDisplay(o)) {
	            p.display(o);
		    displayed = true;
		    break;	// use only the first plugin to display o
	        }
	    if (!displayed) // no plugin suitable for o
	    	System.err.println("no plugin suitable for "+o);
	}
    }
}
